@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010-2011
  * Martin Lambers <marlam@marlam.de>
+ * Gabriele Greco <gabrielegreco@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,24 +22,54 @@
 #ifndef AUDIO_OUTPUT_H
 #define AUDIO_OUTPUT_H
 
-#include <cstddef>
-#include <stdint.h>
+#include <vector>
+#include <string>
 
-#include "decoder.h"
+#include <AL/al.h>
+#include <AL/alc.h>
+
+#include "media_data.h"
 #include "controller.h"
 
 
 class audio_output : public controller
 {
+private:
+    // Static configuration
+    static const size_t _num_buffers;   // Number of audio buffers
+    static const size_t _buffer_size;   // Size of each audio buffer
+
+    // OpenAL things
+    bool _initialized;                  // Was this initialized?
+    ALCdevice *_device;                 // Audio device        
+    ALCcontext *_context;               // Audio context associated with device
+    std::vector<ALuint> _buffers;       // Buffer handles
+    ALuint _source;                     // Audio source
+    ALint _state;                       // State of audio source
+
+    // Properties of the audio data in the current buffers
+    std::vector<int64_t> _buffer_channels;      // Number of channels
+    std::vector<int64_t> _buffer_sample_bits;   // Number of sample bits
+    std::vector<int64_t> _buffer_rates;         // Sample rate in Hz
+
+    // Time management
+    int64_t _past_time;                 // Time that represents all finished buffers
+    int64_t _last_timestamp;            // 
+    int64_t _ext_timer_at_last_timestamp;
+    int64_t _last_reported_timestamp;
+
+    // Get an OpenAL source format for the audio data in blob (or throw an exception)
+    ALenum get_al_format(const audio_blob &blob);
+
 public:
-    audio_output(bool receive_notifications = true) throw ();
-    virtual ~audio_output();
+    audio_output(bool receive_notifications = false);
+    ~audio_output();
+    
+    /* Initialize the audio device for output. Throw an exception if this fails. */
+    void init();
+    /* Deinitialize the audio device. */
+    void deinit();
 
-    /* Open an audio device for output of audio data with the given specifications.
-     * Rate is in samples per second, channels is one of 1 (mono), 2 (stereo), 4 (quad),
-     * 6 (5:1), 7 (6:1), or 8 (7:1), and format is one of the formats defined by the decoder. */
-    virtual void open(int channels, int rate, enum decoder::audio_sample_format format) = 0;
-   
     /* To play audio, do the following:
      * - First, call status() to find out the initial amount of audio data that
      *   is required. The returned time stamp is meaningless on this first call.
@@ -51,19 +82,16 @@ public:
      *   status() minus the start time that the start() function returned. You can also
      *   just query the time without asking if more data is needed by passing NULL as
      *   required_data. */
-    virtual int64_t status(size_t *required_data) = 0;
-    virtual void data(const void *buffer, size_t size) = 0;
-    virtual int64_t start() = 0;
+    int64_t status(size_t *required_data);
+    void data(const audio_blob &blob);
+    int64_t start();
 
     /* Pause/unpause audio playback. */
-    virtual void pause() = 0;
-    virtual void unpause() = 0;
+    void pause();
+    void unpause();
 
     /* Stop audio playback, and flush all buffers. */
-    virtual void stop() = 0;
-
-    /* Finally, clean up and close the device. */
-    virtual void close() = 0;
+    void stop();
 };
 
 #endif
